from .timetable import TimetableFactory, Category
from datetime import datetime, date, time, timedelta
import pickle
import unittest


class TestTimetableFactory(unittest.TestCase):
    # Timetable factory tests
    # These tests were written as if the current date was the 1 of January,
    # 2001 (they are automatically adapted to the current date).
    # The weeks stored in the timetable are:
    #  0 25/12/2000 to 31/12/2000 (previous week)
    #  1 01/01/2001 to 07/01/2001 (current week)
    #  2 08/01/2001 to 14/01/2001 (next week)
    #  …
    # 61 25/02/2002 to 03/03/2002 (last week)

    def __init__(self, name):
        super(TestTimetableFactory, self).__init__(name)
        prev_monday = date.today()
        prev_monday -= timedelta(days=prev_monday.weekday())
        self.offset = prev_monday - date(2001, 1, 1)
        self.factory = None
        self.maxDiff = None

    def setUp(self):
        self.factory = TimetableFactory("Test")

    def test_before(self):
        start = datetime(2000, 12, 2, 10, 0, 0)
        end = datetime(2000, 12, 24, 23, 59, 59)
        self.add_event(start, end)
        self.check_timetable({})

        self.assertEqual(len(self.factory.table._events), 0)

    def test_straddling_start(self):
        start = datetime(2000, 12, 2, 10, 0, 0)
        end = datetime(2000, 12, 25, 8, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                0: [[date(2000, 12, 25), None, time(8, 0, 0)]],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_border_start(self):
        start = datetime(2000, 12, 25, 0, 0, 0)
        end = datetime(2000, 12, 25, 8, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                0: [[date(2000, 12, 25), time(0, 0, 0), time(8, 0, 0)]],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_inside_single_day_1(self):
        start = datetime(2001, 1, 10, 8, 0, 0)
        end = datetime(2001, 1, 10, 10, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                2: [[date(2001, 1, 10), time(8, 0, 0), time(10, 0, 0)]],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_inside_single_day_2(self):
        start = datetime(2001, 1, 10, 8, 0, 0)
        end = datetime(2001, 1, 11, 0, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                2: [[date(2001, 1, 10), time(8, 0, 0), None]],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_inside_multiple_days_1(self):
        start = datetime(2001, 1, 10, 8, 0, 0)
        end = datetime(2001, 1, 12, 10, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                2: [
                    [date(2001, 1, 10), time(8, 0, 0), None],
                    [date(2001, 1, 11), None, None],
                    [date(2001, 1, 12), None, time(10, 0, 0)],
                ],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_inside_multiple_days_2(self):
        start = datetime(2001, 1, 10, 8, 0, 0)
        end = datetime(2001, 1, 13, 0, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                2: [
                    [date(2001, 1, 10), time(8, 0, 0), None],
                    [date(2001, 1, 11), None, None],
                    [date(2001, 1, 12), None, None],
                ],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_multiple_events(self):
        start = datetime(2001, 1, 1, 8, 0, 0)
        end = datetime(2001, 1, 1, 11, 0, 0)
        self.add_event(start, end)

        start = datetime(2001, 1, 10, 8, 0, 0)
        end = datetime(2001, 1, 12, 10, 0, 0)
        self.add_event(start, end)

        start = datetime(2001, 1, 10, 9, 0, 0)
        end = datetime(2001, 1, 10, 11, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                1: [
                    [date(2001, 1, 1), time(8, 0, 0), time(11, 0, 0)],
                ],
                2: [
                    [date(2001, 1, 10), time(8, 0, 0), None],
                    [date(2001, 1, 11), None, None],
                    [date(2001, 1, 12), None, time(10, 0, 0)],
                    [date(2001, 1, 10), time(9, 0, 0), time(11, 0, 0)],
                ],
            }
        )

        self.assertEqual(len(self.factory.table._events), 3)

    def test_inside_straddling_weeks(self):
        start = datetime(2001, 1, 10, 8, 0, 0)
        end = datetime(2001, 1, 23, 10, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                2: [
                    [date(2001, 1, 10), time(8, 0, 0), None],
                    [date(2001, 1, 11), None, None],
                    [date(2001, 1, 12), None, None],
                    [date(2001, 1, 13), None, None],
                    [date(2001, 1, 14), None, None],
                ],
                3: [
                    [date(2001, 1, 15), None, None],
                    [date(2001, 1, 16), None, None],
                    [date(2001, 1, 17), None, None],
                    [date(2001, 1, 18), None, None],
                    [date(2001, 1, 19), None, None],
                    [date(2001, 1, 20), None, None],
                    [date(2001, 1, 21), None, None],
                ],
                4: [
                    [date(2001, 1, 22), None, None],
                    [date(2001, 1, 23), None, time(10, 0, 0)],
                ],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_overflow(self):
        start = datetime(2000, 12, 20, 8, 0, 0)
        end = datetime(2002, 3, 23, 10, 0, 0)
        self.add_event(start, end)

        expected = {}
        cur = date(2000, 12, 25)
        for i in range(62):
            week = []

            for _ in range(7):
                week.append([cur, None, None])
                cur += timedelta(days=1)

            expected[i] = week

        self.check_timetable(expected)

        self.assertEqual(len(self.factory.table._events), 1)

    def test_full(self):
        start = datetime(2000, 12, 25, 0, 0, 0)
        end = datetime(2002, 3, 4, 0, 0, 0)
        self.add_event(start, end)

        expected = {}
        cur = date(2000, 12, 25)
        for i in range(62):
            week = []

            for _ in range(7):
                week.append([cur, None, None])
                cur += timedelta(days=1)

            expected[i] = week

        expected[0][0][1] = time(0, 0, 0)

        self.check_timetable(expected)

        self.assertEqual(len(self.factory.table._events), 1)

    def test_border_end(self):
        start = datetime(2002, 3, 3, 8, 0, 0)
        end = datetime(2002, 3, 4, 0, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                61: [[date(2002, 3, 3), time(8, 0, 0), None]],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_straddling_end(self):
        start = datetime(2002, 3, 3, 8, 0, 0)
        end = datetime(2002, 3, 28, 10, 0, 0)
        self.add_event(start, end)

        self.check_timetable(
            {
                61: [[date(2002, 3, 3), time(8, 0, 0), None]],
            }
        )

        self.assertEqual(len(self.factory.table._events), 1)

    def test_after(self):
        start = datetime(2002, 3, 11, 0, 0, 0)
        end = datetime(2002, 3, 28, 10, 0, 0)
        self.add_event(start, end)

        self.check_timetable({})

        self.assertEqual(len(self.factory.table._events), 0)

    def test_pickle_full(self):
        start = datetime(2000, 12, 25, 0, 0, 0)
        end = datetime(2002, 3, 10, 0, 0, 0)
        self.add_event(start, end)

        table_orig = self.factory.table
        data = pickle.dumps(table_orig, protocol=pickle.HIGHEST_PROTOCOL)
        table_new = pickle.loads(data)

        self.assertEqual(table_orig.title, table_new.title)
        self.assertEqual(table_orig.reason, table_new.reason)
        self.assertEqual(table_orig._weeks, table_new._weeks)

        self.assertEqual(len(self.factory.table._events), 1)

    def add_event(self, start, end):
        start = start + self.offset
        end = end + self.offset

        self.factory.add_event(start, end, "Test", None, Category.AUTRE, [], [], "x")

    def check_timetable(self, expected):
        actual = {}

        for i, week in enumerate(self.factory.table._weeks):
            events = []
            for event in week:
                actual_date = event.date
                if actual_date is not None:
                    actual_date -= self.offset

                events.append([actual_date, event.start, event.end])

            if events:
                actual[i] = events

        self.assertEqual(actual, expected)


if __name__ == "__main__":
    unittest.main(verbosity=2)
